function [HA, Zcenter, Zdelta, B1, B2, B3, c1, c2, c3] = initPowerTrain()
% updated: 21-September-2011, MA

dim=11;

%set parameters of the powertrain
p.alpha = 0.03; %backlash size (half gap) [rad]
p.tau_eng = 0.1; %time constant of the engine [s]
p.b_l = 5.6; %viscous friction of wheels [Nm/(rad/s)]
p.b_m = 0.02; %viscous friction of engine [Nm/(rad/s)]
p.b_i = 1; %viscous friction of additional inertias [Nm/(rad/s)]
p.i = 12; %transmission ratio, Theta_m/Theta_1 [rad/rad]
p.k = 10e3; %shaft stiffness [Nm/rad]
p.k_i = 10e4; %shaft stiffness of additional inertias [Nm/rad]
%p.k_i = 10e3; %shaft stiffness of additional inertias [Nm/rad]
p.r = 0.33; %wheel radius [m]
p.J_l = 140; %moment of inertia of wheels and vehicle mass [kgm^2]
p.J_m = 0.3; %moment of inertia of engine flywheel [kgm^2]
p.J_i = 0.01; %moment of inertia of additional inertias [kgm^2]

%control parameters:
p.k_P = 0;
p.k_I = 0;
p.k_D = 0;
p.k_K = 0.5;
p.k_KD = 0.5;
p.k_KI = 0.5;
p.k_KK = 0;

%create initial zonotope
omega_ref_center = 30;
omega_ref_delta = 10;
T_l_center = -300;

%initial set for loc 3:
x1_0 = p.b_l/p.k*omega_ref_center - p.alpha + T_l_center/p.k;
delta_x1_0 = p.b_l/p.k*omega_ref_delta;
xy_0 = p.b_l/p.k_i*omega_ref_center + T_l_center/p.k_i;
delta_xy_0 = p.b_l/p.k_i*omega_ref_delta;

p.T_m0 = p.k/p.i*(x1_0 + p.alpha) + p.b_m*omega_ref_center; %needs to be integrated in U
delta_T_m0 = p.k/p.i*delta_x1_0 + p.b_m*omega_ref_delta; %needs to be integrated in U

Zcenter = [x1_0;...
           p.T_m0;...
           0;...
           omega_ref_center;...
           0;...
           omega_ref_center;...
           p.i*omega_ref_center; ...
           xy_0; ...
           omega_ref_center; ...
           xy_0; ...
           omega_ref_center];
Zdelta =  [delta_x1_0;...
           delta_T_m0;...
           0;...
           omega_ref_delta; ...
           0; ...
           omega_ref_delta; ...
           p.i*omega_ref_delta;...   
           delta_xy_0; ...
           omega_ref_delta; ...
           delta_xy_0; ...
           omega_ref_delta];    


% %shrink initial zonotope
% Zinit = enlarge(Zinit,0.05*ones(dim,1));

%obtain linear system 
syms x1 x2 x3 x4 x5 x6 x7 x8 x9 x10 x11
syms u1 u2 
syms t

%combine symbolic variables to vectors
x = [x1; x2; x3; x4; x5; x6; x7; x8; x9; x10; x11];
u = [u1; u2];

%compute jacobians for loc 1:
f = powertrain11Eq(t,x,u,p);
c1 = powertrain11Eq(0,zeros(length(x),1),zeros(length(u),1),p); 
A1 = double(jacobian(f, x));
B1 = double(jacobian(f, u));


%compute jacobians for loc 2:
p.kTmp = p.k;
p.k = 0;

f = powertrain11Eq(t,x,u,p);
c2 = powertrain11Eq(0,zeros(length(x),1),zeros(length(u),1),p);
    
A2 = double(jacobian(f, x));
B2 = double(jacobian(f, u));
p.k = p.kTmp;

%compute jacobians for loc 3:
p.alphaTmp = p.alpha;
p.alpha = -p.alpha;

f = powertrain11Eq(t,x,u,p);
c3 = powertrain11Eq(0,zeros(length(x),1),zeros(length(u),1),p);   

A3 = double(jacobian(f, x));
B3 = double(jacobian(f, u));
p.alpha = p.alphaTmp;


%specify linear systems
linSys1 = linearSys('linearSys',A1,eye(dim));
linSys2 = linearSys('linearSys',A2,eye(dim));
linSys3 = linearSys('linearSys',A3,eye(dim));

%define distant
dist = 1e3;
eps = 1e-6;

%specify hyperplanes
%1st
n1 = [-1; zeros(dim-1,1)];
d1 = -p.alpha;
h1 = halfspace(n1,d1); 

n1b = [-1; zeros(dim-1,1)];
d1b = -p.alpha - 1e3;
h1b = halfspace(n1b,d1b); 

%2nd
n2 = [-1; zeros(dim-1,1)];
d2 = p.alpha;
h2 = halfspace(n2,d2); 

n2b = [-1; zeros(dim-1,1)];
d2b = p.alpha + 1e3;
h2b = halfspace(n2b,d2b); 

%loc1:
%invariant
inv = interval([p.alpha; -dist*ones(dim-1,1)], dist*ones(dim,1));
%guard sets
guard1 = h1b;
%resets
reset1.A = eye(dim);
reset1.b = zeros(dim,1);
%transitions
trans{1} = transition(guard1,reset1,2,'a','b'); %--> next loc: 2
%specify location
loc{1} = location('loc1',1,inv,trans,linSys1);

%loc2:
%invariant
inv = interval([-p.alpha - eps; -dist*ones(dim-1,1)], [p.alpha + eps; dist*ones(dim-1,1)]);
%guard set 1
guard1 = h1;
guard2 = h2b;
%reset 1
reset1.A=eye(dim);
reset1.b=zeros(dim,1);
%transition 1
trans=[];
trans{1}=transition(guard1,reset1,1,'a','b'); %--> next loc: 1
trans{2}=transition(guard2,reset1,3,'a','b'); %--> next loc: 3
%specify location
loc{2}=location('loc2',2,inv,trans,linSys2);

%loc3:
%invariant
inv = interval(-dist*ones(dim,1), [-p.alpha; dist*ones(dim-1,1)]);
%guard set 1
guard1 = h2;
%reset 1
reset1.A=eye(dim);
reset1.b=zeros(dim,1);
%transition 1
trans=[];
trans{1}=transition(guard1,reset1,2,'a','b'); %--> next loc: 2
%specify location
loc{3}=location('loc3',3,inv,trans,linSys3);

%specify hybrid automaton
HA=hybridAutomaton(loc);




